@@ -422,7 +422,9 @@ g_hash_table_iter_next (GHashTableIter *iter,
   RealIter *ri = (RealIter *) iter;
 
   g_return_val_if_fail (iter != NULL, FALSE);
+#ifndef G_DISABLE_ASSERT
   g_return_val_if_fail (ri->version == ri->hash_table->version, FALSE);
+#endif
 
   if (ri->pre_advanced)
     {
@@ -484,7 +486,9 @@ iter_remove_or_steal (RealIter *ri, gboolean notify)
   int position;
 
   g_return_if_fail (ri != NULL);
+#ifndef G_DISABLE_ASSERT
   g_return_if_fail (ri->version == ri->hash_table->version);
+#endif
   g_return_if_fail (ri->node != NULL);
 
   prev = ri->prev_node;
@@ -786,7 +790,7 @@ g_hash_table_insert (GHashTable *hash_table,
                      gpointer    key,
                      gpointer    value)
 {
-  return g_hash_table_insert_internal (hash_table, key, value, FALSE);
+  g_hash_table_insert_internal (hash_table, key, value, FALSE);
 }
 
 /**
@@ -807,7 +811,7 @@ g_hash_table_replace (GHashTable *hash_table,
                       gpointer    key,
                       gpointer    value)
 {
-  return g_hash_table_insert_internal (hash_table, key, value, TRUE);
+  g_hash_table_insert_internal (hash_table, key, value, TRUE);
 }
 
 /*
